@@ -6,7 +6,6 @@
 #  (at your option) any later version.
 #
 #  File name: discrete.py
-#  Last modified: 11/15/20, 3:29 PM
 
 import logging
 from typing import List, Tuple, Union
@@ -48,8 +47,9 @@ class Discrete:
         # default minimum iteration number and error tolerance to allow checking
         # To enable `min_iter` and `err_tol`, a `Discrete` subclass needs to call
         # `check_iter_err()` manually in `check_var()` and/or `check_eq()`.
+
         self.min_iter = min_iter
-        self.err_tol = 1e-2
+        self.err_tol = err_tol
 
         self.has_check_var = False  # if subclass implements `check_var()`
         self.has_check_eq = False   # if subclass implements `check_eq()`
@@ -59,10 +59,20 @@ class Discrete:
         This function is called in ``l_update_var`` before evaluating equations.
 
         It should update internal flags only.
+
+        Parameters
+        ----------
+        adjust_upper : bool
+            True to adjust the upper limit to the value of the variable.
+            Supported by limiters.
+        adjust_lower : bool
+            True to adjust the lower limit to the value of the variable.
+            Supported by limiters.
+
         """
         pass
 
-    def check_eq(self):
+    def check_eq(self, **kwargs):
         """
         This function is called in ``l_check_eq`` after updating equations.
 
@@ -108,41 +118,94 @@ class Discrete:
         return self.__class__.__name__
 
     def list2array(self, n):
+        """
+        Allocate memory for the discrete flags specified in `self.export_flags`.
+
+        Parameters
+        ----------
+        n : int
+            Number of elements in the array. Provided by the calling function.
+        """
+
         for flag in self.export_flags:
             self.__dict__[flag] = self.__dict__[flag] * np.ones(n, dtype=float)
 
     def warn_init_limit(self):
         """
-        Warn if initialized at limits.
+        Warn if associated variables are initialized at limits.
         """
+
         if self.no_warn:
             return
 
         for f, limit in self.warn_flags:
             if f not in self.export_flags:
-                logger.error(f'warn_flags contain unknown flag {f}')
+                logger.error('warn_flags contain unknown flag %s', f)
                 continue
 
-            pos = np.argwhere(np.not_equal(self.__dict__[f], 0)).ravel()
-            if not len(pos):
-                continue
-            err_msg = f'{self.owner.class_name}.{self.name} at limits <{self.__dict__[limit].name}>'
-            if isinstance(self.__dict__[limit].v, np.ndarray):
-                lim_value = self.__dict__[limit].v[pos]
+            mask = np.ones(self.owner.n, dtype=bool)
+            if limit == 'upper':
+                mask = self.mask_upper
+            elif limit == 'lower':
+                mask = self.mask_lower
             else:
+                logger.debug('Unknown limit name <%s>', limit)
+
+            # process online devices only
+            flag_vals = np.logical_and(self.__dict__[f], self.owner.u.v)
+
+            # ignore limits that has been adjusted
+            flag_vals = np.logical_and(flag_vals, np.logical_not(mask))
+
+            pos = np.argwhere(np.not_equal(flag_vals, 0)).ravel()
+
+            if len(pos) == 0:
+                continue
+
+            # convert limie values to arrays
+            if isinstance(self.__dict__[limit].v, np.ndarray):
                 lim_value = self.__dict__[limit].v
+            else:
+                lim_value = self.__dict__[limit].v * np.ones(self.owner.n)
 
-            err_data = {'idx': [self.owner.idx.v[i] for i in pos],
-                        'Flag': [f] * len(pos),
-                        'Input Value': self.u.v[pos],
-                        'Limit': lim_value * np.ones_like(pos),
-                        }
+            at_limit_pos = list()
+            out_limit_pos = list()
 
-            tab = Tab(title=err_msg,
-                      header=err_data.keys(),
-                      data=list(map(list, zip(*err_data.values()))))
+            for item in pos:
+                if np.isclose(lim_value[item], self.u.v[item]):
+                    at_limit_pos.append(item)
+                else:
+                    out_limit_pos.append(item)
 
-            logger.warning(tab.draw())
+            if len(out_limit_pos) > 0:
+                # warn out of limits
+                err_msg = f'{self.owner.class_name}.{self.name} out of limits <{self.__dict__[limit].name}>'
+                err_data = {'idx': [self.owner.idx.v[i] for i in out_limit_pos],
+                            'Flag': [f] * len(out_limit_pos),
+                            'Input Value': self.u.v[out_limit_pos],
+                            'Limit': lim_value[out_limit_pos]
+                            }
+
+                tab = Tab(title=err_msg,
+                          header=err_data.keys(),
+                          data=list(map(list, zip(*err_data.values()))))
+
+                logger.warning(tab.draw())
+
+            if len(at_limit_pos) > 0:
+                # warn at limits
+                err_msg = f'{self.owner.class_name}.{self.name} at limits <{self.__dict__[limit].name}>'
+                err_data = {'idx': [self.owner.idx.v[i] for i in at_limit_pos],
+                            'Flag': [f] * len(at_limit_pos),
+                            'Input Value': self.u.v[at_limit_pos],
+                            'Limit': lim_value[at_limit_pos]
+                            }
+
+                tab = Tab(title=err_msg,
+                          header=err_data.keys(),
+                          data=list(map(list, zip(*err_data.values()))))
+
+                logger.debug(tab.draw())
 
     def check_iter_err(self, niter=None, err=None):
         """
@@ -173,7 +236,7 @@ class Discrete:
 
 class LessThan(Discrete):
     """
-    Less than (<) comparison function.
+    Less than (<) comparison function that tests if ``u < bound``.
 
     Exports two flags: z1 and z0.
     For elements satisfying the less-than condition, the corresponding z1 = 1.
@@ -182,11 +245,14 @@ class LessThan(Discrete):
     Notes
     -----
     The default z0 and z1, if not enabled, can be set through the constructor.
+    By default, the model will not adjust the limit.
     """
 
     def __init__(self, u, bound, equal=False, enable=True, name=None, tex_name=None,
-                 info=None, cache=False, z0=0, z1=1):
+                 info: str = None, cache: bool = False,
+                 z0=0, z1=1):
         super().__init__(name=name, tex_name=tex_name, info=info)
+
         self.u = u
         self.bound = dummify(bound)
         self.equal: bool = equal
@@ -205,6 +271,7 @@ class LessThan(Discrete):
         """
         If enabled, set flags based on inputs. Use cached values if enabled.
         """
+
         if not self.enable:
             return
 
@@ -269,8 +336,10 @@ class Limiter(Discrete):
         Flags for violating the upper limit
     """
 
-    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None,
+    def __init__(self, u, lower, upper, enable=True,
+                 name: str = None, tex_name: str = None, info: str = None,
                  min_iter: int = 2, err_tol: float = 0.01,
+                 allow_adjust: bool = True,
                  no_lower=False, no_upper=False, sign_lower=1, sign_upper=1,
                  equal=True, no_warn=False,
                  zu=0.0, zl=0.0, zi=1.0):
@@ -283,6 +352,8 @@ class Limiter(Discrete):
         self.no_lower = no_lower
         self.no_upper = no_upper
 
+        self.allow_adjust = allow_adjust
+
         if sign_lower not in (1, -1):
             raise ValueError("sign_lower must be 1 or -1, got %s" % sign_lower)
         if sign_upper not in (1, -1):
@@ -297,6 +368,9 @@ class Limiter(Discrete):
         self.zl = np.array([zl])
         self.zi = np.array([zi])
 
+        self.mask_upper = None
+        self.mask_lower = None
+
         self.has_check_var = True
 
         self.export_flags.append('zi')
@@ -311,15 +385,25 @@ class Limiter(Discrete):
             self.export_flags_tex.append('z_u')
             self.warn_flags.append(('zu', 'upper'))
 
-    def check_var(self, *args, **kwargs):
+    def check_var(self,
+                  allow_adjust=True,  # allow flag from model
+                  adjust_lower=False,
+                  adjust_upper=False,
+                  *args, **kwargs):
         """
-        Evaluate the flags.
+        Check the input variable and set flags.
         """
+
         if not self.enable:
             return
 
         if not self.no_upper:
             upper_v = -self.upper.v if self.sign_upper.v == -1 else self.upper.v
+
+            # FIXME: adjust will not be successful when sign is -1
+            if self.allow_adjust and allow_adjust and adjust_upper:
+                self.do_adjust_upper(self.u.v, upper_v, allow_adjust, adjust_upper)
+
             if self.equal:
                 self.zu[:] = np.greater_equal(self.u.v, upper_v)
             else:
@@ -327,6 +411,11 @@ class Limiter(Discrete):
 
         if not self.no_lower:
             lower_v = -self.lower.v if self.sign_lower.v == -1 else self.lower.v
+
+            # FIXME: adjust will not be successful when sign is -1
+            if self.allow_adjust and allow_adjust and adjust_lower:
+                self.do_adjust_lower(self.u.v, lower_v, allow_adjust, adjust_lower)
+
             if self.equal:
                 self.zl[:] = np.less_equal(self.u.v, lower_v)
             else:
@@ -334,6 +423,69 @@ class Limiter(Discrete):
 
         self.zi[:] = np.logical_not(np.logical_or(self.zu, self.zl))
 
+    def do_adjust_lower(self, val, lower, allow_adjust=True, adjust_lower=False):
+        """
+        Adjust the lower limit.
+
+        Notes
+        -----
+        This method is only available if `allow_adjust` is True
+        and `adjust_lower` is True.
+        """
+
+        if allow_adjust:
+            mask = (val < lower)
+
+            if sum(mask) == 0:
+                return
+
+            if adjust_lower:
+                self._show_adjust(val, lower, mask, self.lower.name, adjusted=True)
+                lower[mask] = val[mask]
+                self.mask_lower = mask  # store after adjusting
+            else:
+                self._show_adjust(val, lower, mask, self.lower.name, adjusted=False)
+
+    def _show_adjust(self, val, old_limit, mask, limit_name, adjusted=True):
+        """
+        Helper function to show a table of the adjusted limits.
+        """
+        idxes = np.array(self.owner.idx.v)[mask]
+
+        adjust_or_not = 'adjusted' if adjusted else 'unadjusted'
+
+        tab = Tab(title=f"{self.owner.class_name}.{self.name}: {adjust_or_not} limit <{limit_name}>",
+                  header=['Idx', 'Input', 'Old Limit'],
+                  data=[*zip(idxes, val[mask], old_limit[mask])],
+                  )
+
+        if adjusted:
+            logger.info(tab.draw())
+        else:
+            logger.warning(tab.draw())
+
+    def do_adjust_upper(self, val, upper, allow_adjust=True, adjust_upper=False):
+        """
+        Adjust the upper limit.
+
+        Notes
+        -----
+        This method is only available if `allow_adjust` is True
+        and `adjust_upper` is True.
+        """
+
+        if allow_adjust:
+            mask = (val > upper)
+            if sum(mask) == 0:
+                return
+
+            if adjust_upper:
+                self._show_adjust(val, upper, mask, self.upper.name, adjusted=True)
+                upper[mask] = val[mask]
+                self.mask_upper = mask
+            else:
+                self._show_adjust(val, upper, mask, self.lower.name, adjusted=False)
+
 
 class SortedLimiter(Limiter):
     """
@@ -361,12 +513,14 @@ class SortedLimiter(Limiter):
     def __init__(self, u, lower, upper, n_select: int = 5,
                  name=None, tex_name=None, enable=True, abs_violation=True,
                  min_iter: int = 2, err_tol: float = 0.01,
+                 allow_adjust: bool = True,
                  zu=0.0, zl=0.0, zi=1.0, ql=0.0, qu=0.0,
                  ):
 
         super().__init__(u, lower, upper,
                          enable=enable, name=name, tex_name=tex_name,
                          min_iter=min_iter, err_tol=err_tol,
+                         allow_adjust=allow_adjust,
                          zu=zu, zl=zl, zi=zi,
                          )
 
@@ -512,11 +666,13 @@ class AntiWindup(Limiter):
 
     def __init__(self, u, lower, upper, enable=True, no_warn=False,
                  no_lower=False, no_upper=False, sign_lower=1, sign_upper=1,
-                 name=None, tex_name=None, info=None, state=None):
+                 name=None, tex_name=None, info=None, state=None,
+                 allow_adjust: bool = True,):
         super().__init__(u, lower, upper, enable=enable, no_warn=no_warn,
                          no_lower=no_lower, no_upper=no_upper,
                          sign_lower=sign_lower, sign_upper=sign_upper,
-                         name=name, tex_name=tex_name, info=info)
+                         name=name, tex_name=tex_name, info=info,
+                         allow_adjust=allow_adjust,)
         self.state = state if state else u
 
         self.has_check_var = False
@@ -529,7 +685,11 @@ class AntiWindup(Limiter):
         """
         pass
 
-    def check_eq(self):
+    def check_eq(self,
+                 allow_adjust=True,
+                 adjust_lower=False,
+                 adjust_upper=False,
+                 **kwargs):
         """
         Check the variables and equations and set the limiter flags.
         Reset differential equation values based on limiter flags.
@@ -541,11 +701,21 @@ class AntiWindup(Limiter):
         """
         if not self.no_upper:
             upper_v = -self.upper.v if self.sign_upper.v == -1 else self.upper.v
+
+            if self.allow_adjust and allow_adjust and adjust_upper:
+                self.do_adjust_upper(self.u.v, upper_v,
+                                     allow_adjust=allow_adjust,
+                                     adjust_upper=adjust_upper)
             self.zu[:] = np.logical_and(np.greater_equal(self.u.v, upper_v),
                                         np.greater_equal(self.state.e, 0))
 
         if not self.no_lower:
             lower_v = -self.lower.v if self.sign_lower.v == -1 else self.lower.v
+
+            if self.allow_adjust and allow_adjust and adjust_lower:
+                self.do_adjust_lower(self.u.v, lower_v,
+                                     allow_adjust=allow_adjust,
+                                     adjust_lower=adjust_lower)
             self.zl[:] = np.logical_and(np.less_equal(self.u.v, lower_v),
                                         np.less_equal(self.state.e, 0))
 
@@ -605,6 +775,9 @@ class RateLimiter(Discrete):
         self.rate_lower_cond = dummify(lower_cond)
         self.rate_upper_cond = dummify(upper_cond)
 
+        self.mask_lower = None
+        self.mask_upper = None
+
         self.rate_no_lower = no_lower
         self.rate_no_upper = no_upper
         self.enable = enable
@@ -628,7 +801,7 @@ class RateLimiter(Discrete):
             self.export_flags_tex.append('z_{ur}')
             self.warn_flags.append(('zur', 'upper'))
 
-    def check_eq(self):
+    def check_eq(self, **kwargs):
         if not self.enable:
             return
 
@@ -658,7 +831,9 @@ class AntiWindupRate(AntiWindup, RateLimiter):
     def __init__(self, u, lower, upper, rate_lower, rate_upper,
                  no_lower=False, no_upper=False, rate_no_lower=False, rate_no_upper=False,
                  rate_lower_cond=None, rate_upper_cond=None,
-                 enable=True, name=None, tex_name=None, info=None):
+                 enable=True, name=None, tex_name=None, info=None,
+                 allow_adjust: bool = True,):
+
         RateLimiter.__init__(self, u, lower=rate_lower, upper=rate_upper, enable=enable,
                              no_lower=rate_no_lower, no_upper=rate_no_upper,
                              lower_cond=rate_lower_cond, upper_cond=rate_upper_cond,
@@ -667,11 +842,12 @@ class AntiWindupRate(AntiWindup, RateLimiter):
         AntiWindup.__init__(self, u, lower=lower, upper=upper, enable=enable,
                             no_lower=no_lower, no_upper=no_upper,
                             name=name, tex_name=tex_name, info=info,
+                            allow_adjust=allow_adjust,
                             )
 
-    def check_eq(self):
-        RateLimiter.check_eq(self)
-        AntiWindup.check_eq(self)
+    def check_eq(self, **kwargs):
+        RateLimiter.check_eq(self, **kwargs)
+        AntiWindup.check_eq(self, **kwargs)
 
 
 class Selector(Discrete):
@@ -798,8 +974,8 @@ class Switcher(Discrete):
     """
 
     def __init__(self, u, options: Union[list, Tuple], info: str = None,
-                 name: str = None, tex_name: str = None, cache=True):
-        super().__init__(name=name, tex_name=tex_name, info=info)
+                 name: str = None, tex_name: str = None, cache=True,):
+        super().__init__(name=name, tex_name=tex_name, info=info,)
         self.u = u
         self.options: Union[List, Tuple] = options
         self.cache: bool = cache
@@ -888,10 +1064,16 @@ class DeadBand(Limiter):
 
     """
 
-    def __init__(self, u, center, lower, upper, enable=True, equal=False, zu=0.0, zl=0.0, zi=0.0,
-                 name=None, tex_name=None, info=None):
-        Limiter.__init__(self, u, lower, upper, enable=enable, equal=equal, zi=zi, zl=zl, zu=zu,
-                         name=name, tex_name=tex_name, info=info)
+    def __init__(self, u, center, lower, upper,
+                 enable=True, equal=False,
+                 zu=0.0, zl=0.0, zi=0.0,
+                 name=None, tex_name=None, info=None,
+                 ):
+        Limiter.__init__(self, u, lower, upper,
+                         enable=enable, equal=equal, zi=zi, zl=zl, zu=zu,
+                         name=name, tex_name=tex_name, info=info,
+                         allow_adjust=False,)
+
         self.center = dummify(center)  # CURRENTLY NOT IN USE
 
     def check_var(self, *args, **kwargs):
@@ -1016,7 +1198,9 @@ class Delay(Discrete):
 
     """
 
-    def __init__(self, u, mode='step', delay=0, name=None, tex_name=None, info=None):
+    def __init__(self, u, mode='step', delay=0,
+                 name=None, tex_name=None, info=None):
+
         Discrete.__init__(self, name=name, tex_name=tex_name, info=info)
 
         if mode not in ('step', 'time'):
