@@ -11,6 +11,7 @@ Base class for building ANDES models.
 #  File name: model.py
 
 import logging
+import pprint
 from collections import OrderedDict
 from typing import Iterable, Sized, Union, Callable
 
@@ -31,6 +32,7 @@ from andes.core.symprocessor import SymProcessor
 from andes.core.var import Algeb, BaseVar, ExtAlgeb, ExtState, State
 from andes.shared import jac_full_names, jac_names, jac_types, np, pd, numba
 from andes.utils.func import list_flatten
+from andes.utils.tab import Tab
 
 logger = logging.getLogger(__name__)
 
@@ -646,6 +648,24 @@ class Model:
         if config is not None:
             self.config.load(config)
 
+        # basic configs
+        self.config.add(OrderedDict((('allow_adjust', 1),
+                                    ('adjust_lower', 0),
+                                    ('adjust_upper', 1),
+                                     )))
+
+        self.config.add_extra("_help",
+                              allow_adjust='allow adjusting upper or lower limits',
+                              adjust_lower='adjust lower limit',
+                              adjust_upper='adjust upper limit',
+                              )
+
+        self.config.add_extra("_alt",
+                              allow_adjust=(0, 1),
+                              adjust_lower=(0, 1),
+                              adjust_upper=(0, 1),
+                              )
+
         self.calls = ModelCall()  # callback and LaTeX string storage
         self.triplets = JacTriplet()  # Jacobian triplet storage
         self.syms = SymProcessor(self)  # symbolic processor instance
@@ -738,7 +758,7 @@ class Model:
 
     def _check_attribute(self, key, value):
         """
-        Check the attribute pair for valid names.
+        Check the attribute pair for valid names while instantiating the class.
 
         This function assigns `owner` to the model itself, assigns the name and tex_name.
         """
@@ -753,6 +773,17 @@ class Model:
                 logger.warning(f"{self.class_name}: redefinition of member <{key}>. Likely a modeling error.")
 
     def __setattr__(self, key, value):
+        """
+        Overload the setattr function to register attributes.
+
+        Parameters
+        ----------
+        key : str
+            name of the attribute
+        value : [BaseVar, BaseService, Discrete, Block]
+            value of the attribute
+        """
+
         self._check_attribute(key, value)
 
         # store the variable declaration order
@@ -1053,7 +1084,7 @@ class Model:
             if instance.has_check_var:
                 instance.check_var(dae_t=dae_t, niter=niter, err=err)
 
-    def l_check_eq(self):
+    def l_check_eq(self, init=False, **kwargs):
         """
         Call the ``check_eq`` method of discrete components to update equation-dependent flags.
 
@@ -1064,9 +1095,17 @@ class Model:
         -------
         None
         """
-        for instance in self.discrete.values():
-            if instance.has_check_eq:
-                instance.check_eq()
+        if init:
+            for instance in self.discrete.values():
+                if instance.has_check_eq:
+                    instance.check_eq(allow_adjust=self.config.allow_adjust,
+                                      adjust_lower=self.config.adjust_lower,
+                                      adjust_upper=self.config.adjust_upper,
+                                      )
+        else:
+            for instance in self.discrete.values():
+                if instance.has_check_eq:
+                    instance.check_eq()
 
     def s_update(self):
         """
@@ -1723,7 +1762,15 @@ class Model:
 
         return f'{self.class_name} ({self.n} {dev_text}) at {hex(id(self))}'
 
-    def init(self, routine):
+    def _log_init_debug(self, *args):
+        """
+        Helper function to log initialization debug message.
+        """
+
+        if self.system.options.get("init") is True:
+            logger.debug(*args)
+
+    def init(self, routine, debug=False):
         """
         Numerical initialization of a model.
 
@@ -1744,40 +1791,102 @@ class Model:
         else:
             do_init = getattr(self.flags, flag_name)
 
-        logger.debug(f'{self.class_name} has {flag_name} = {do_init}')
+        self._log_init_debug('%s has <%s> = %s', self.class_name, flag_name, do_init)
 
         if do_init:
             kwargs = self.get_inputs(refresh=True)
 
+            self._log_init_debug('%s: initialization sequence:', self.class_name)
+            self._log_init_debug(' -> '.join([str(i) for i in self.calls.init_seq]))
+            self._log_init_debug("%s: assignment initialization phase begins", self.class_name)
+
             for idx, name in enumerate(self.calls.init_seq):
                 # single variable - do assignment
                 if isinstance(name, str):
+                    self._log_init_debug("%s: entering <%s> assignment init", self.class_name, name)
                     instance = self.__dict__[name]
-                    _eval_discrete(instance)
+                    if instance.discrete is not None:
+                        self._log_init_debug("%s: evaluate discrete <%s>", name, instance.discrete)
+                        _eval_discrete(instance, self.config.allow_adjust,
+                                       self.config.adjust_lower, self.config.adjust_upper)
+
                     if instance.v_str is not None:
+                        arg_print = OrderedDict()
+                        if self.system.options.get("init"):
+                            for a, b in zip(self.calls.ia_args[name], self.ia_args[name]):
+                                arg_print[a] = b
+
                         if not instance.v_str_add:
                             # assignment is for most variable initialization
+                            self._log_init_debug("%s: new values will be assigned (=)", name)
                             instance.v[:] = self.calls.ia[name](*self.ia_args[name])
+
                         else:
                             # in-place add initial values.
                             # Voltage compensators can set part of the `v` of exciters.
                             # Exciters will then set the bus voltage part.
+                            self._log_init_debug("%s: new values will be in-place added (+=)", name)
                             instance.v[:] += self.calls.ia[name](*self.ia_args[name])
 
+                        arg_print[name] = instance.v
+
+                        if self.system.options.get("init"):
+                            for key, val in arg_print.items():
+                                if isinstance(val, (int, float, np.floating, np.integer)) or \
+                                        isinstance(val, np.ndarray) and val.ndim == 0:
+
+                                    arg_print[key] = val * np.ones_like(instance.v)
+                                if isinstance(val, np.ndarray) and val.dtype == complex:
+                                    arg_print[key] = [str(i) for i in val]
+
+                            tab = Tab(title="v_str of %s is '%s'" % (name, instance.v_str),
+                                      header=["idx", *self.calls.ia_args[name], name],
+                                      data=list(zip(self.idx.v, *arg_print.values())),
+                                      )
+                            self._log_init_debug(tab.draw())
+
                     # single variable iterative solution
                     if name in self.calls.ii:
+                        self._log_init_debug("\n%s: entering <%s> iterative init", self.class_name,
+                                             pprint.pformat(name))
+
                         self.solve_iter(name, kwargs)
 
+                        self._log_init_debug("%s new values are %s", name, self.__dict__[name].v)
+
                 # multiple variables, iterative
                 else:
+                    self._log_init_debug("\n%s: entering <%s> iterative init",
+                                         self.class_name, name)
+
                     for vv in name:
                         instance = self.__dict__[vv]
-                        _eval_discrete(instance)
+
+                        if instance.discrete is not None:
+                            self._log_init_debug("%s: evaluate discrete <%s>",
+                                                 name, instance.discrete)
+
+                            _eval_discrete(instance, self.config.allow_adjust,
+                                           self.config.adjust_lower, self.config.adjust_upper)
                         if instance.v_str is not None:
+                            self._log_init_debug("%s: v_str = %s", vv, instance.v_str)
+
+                            arg_print = OrderedDict()
+                            if self.system.options.get("init"):
+                                for a, b in zip(self.calls.ia_args[vv], self.ia_args[vv]):
+                                    arg_print[a] = b
+                                self._log_init_debug(pprint.pformat(arg_print))
+
                             instance.v[:] = self.calls.ia[vv](*self.ia_args[vv])
 
+                    self._log_init_debug("\n%s: entering <%s> iterative init", self.class_name,
+                                         pprint.pformat(name))
                     self.solve_iter(name, kwargs)
 
+                    for vv in name:
+                        instance = self.__dict__[vv]
+                        self._log_init_debug("%s new values are %s", vv, instance.v)
+
             # call custom variable initializer after generated init
             kwargs = self.get_inputs(refresh=True)
             self.v_numeric(**kwargs)
@@ -1792,6 +1901,9 @@ class Model:
         Solve iterative initialization.
         """
         for pos in range(self.n):
+            logger.debug("%s: iterative init for %s, device pos = %s",
+                         self.class_name, name, pos)
+
             self.solve_iter_single(name, kwargs, pos)
 
     def solve_iter_single(self, name, inputs, pos):
@@ -1818,27 +1930,40 @@ class Model:
         solved = False
 
         while niter < maxiter:
+            logger.debug("iteration %s:", niter)
+
             i_args = [item[pos] for item in ii_args]  # all variables of one device at a time
             j_args = [item[pos] for item in ij_args]
 
             b = np.ravel(rhs(*i_args))
             A = jac(*j_args)
-            inc = - sp.linalg.lu_solve(sp.linalg.lu_factor(A), b)
-            if np.isnan(inc).any():
-                logger.warning(f"{self.class_name}: nan detected in iterations for {self.idx.v[pos]}.")
-                break
 
-            x0 += inc
-            for idx, item in enumerate(name):
-                inputs[item][pos] = x0[idx]
-
-            mis = np.max(np.abs(inc))
-            niter += 1
+            logger.debug("A:\n%s", A)
+            logger.debug("b:\n%s", b)
 
+            mis = np.max(np.abs(b))
             if mis <= eps:
                 solved = True
                 break
 
+            inc = - sp.linalg.lu_solve(sp.linalg.lu_factor(A), b)
+            if np.isnan(inc).any():
+                if self.u.v[pos] != 0:
+                    logger.debug("%s: nan ignored in iterations for offline device pos = %s",
+                                 self.class_name, pos)
+                else:
+                    logger.error("%s: nan error detected in iterations for device pos = %s",
+                                 self.class_name, pos)
+                break
+
+            x0 += inc
+            logger.debug("solved x0:\n%s", x0)
+
+            for idx, item in enumerate(name):
+                inputs[item][pos] = x0[idx]
+
+            niter += 1
+
         if solved:
             for idx, item in enumerate(name):
                 inputs[item][pos] = x0[idx]
@@ -1858,18 +1983,37 @@ class Model:
         pass
 
 
-def _eval_discrete(instance):
-    # for variables associated with limiters, limiters need to be evaluated
-    # before variable initialization.
-    # However, if any limit is hit, initialization is likely to fail.
+def _eval_discrete(instance, allow_adjust=True,
+                   adjust_lower=False, adjust_upper=False):
+    """
+    Evaluate discrete components associated with a variable instance.
+    Calls ``check_var()`` on the discrete components.
 
-    if instance.discrete is not None:
-        if not isinstance(instance.discrete, (list, tuple, set)):
-            dlist = (instance.discrete,)
-        else:
-            dlist = instance.discrete
-        for d in dlist:
-            d.check_var()
+    For variables associated with limiters, limiters need to be evaluated
+    before variable initialization.
+    However, if any limit is hit, initialization is likely to fail.
+
+    Parameters
+    ----------
+    instance : BaseVar
+        instance of a variable
+    allow_adjust : bool, optional
+        True to enable overall adjustment
+    adjust_lower : bool, optional
+        True to adjust lower limits to the input values
+    adjust_upper : bool, optional
+        True to adjust upper limits to the input values
+
+    """
+
+    if not isinstance(instance.discrete, (list, tuple, set)):
+        dlist = (instance.discrete,)
+    else:
+        dlist = instance.discrete
+    for d in dlist:
+        d.check_var(allow_adjust=allow_adjust,
+                    adjust_lower=adjust_lower,
+                    adjust_upper=adjust_upper)
 
 
 def to_jit(func: Union[Callable, None],
