@@ -1,8 +1,7 @@
-from andes.core.param import NumParam
 from andes.core.block import Lag
-
-from andes.models.renewable.regcvsg import REGCVSGData, VSGOuterPIData
-from andes.models.renewable.regcvsg import REGCVSGModelBase, VSGOuterPIModel
+from andes.core.param import NumParam
+from andes.models.renewable.regcvsg import (REGCVSGData, REGCVSGModelBase,
+                                            VSGOuterPIData, VSGOuterPIModel,)
 
 
 class VSGInnerLagData:
@@ -28,7 +27,7 @@ class REGCVSG2(REGCVSGData, VSGOuterPIData, VSGInnerLagData,
     """
     Voltage-controlled VSC with VSG control.
 
-    PI controllers are replaced with lag transfer functions.
+    The inner-loop current PI controllers are replaced with lag transfer functions.
     """
 
     def __init__(self, system, config):
