@@ -12,58 +12,30 @@ This module imports shared libraries either directly or with `LazyImport`.
 
 import math
 import os
-import coloredlogs         # NOQA
-import numpy as np         # NOQA
-
-from andes.utils.lazyimport import LazyImport
 from distutils.spawn import find_executable
 
-# Library preference:
-# KVXOPT + ipadd > CVXOPT + ipadd > KXVOPT > CVXOPT (+ KLU or not)
+import coloredlogs  # NOQA
+import numpy as np  # NOQA
+from kvxopt import printing  # NOQA
+from kvxopt import (div, klu, matrix, mul, sparse, spdiag, spmatrix,  # NOQA
+                    umfpack,)
+from kvxopt.lapack import gesv  # NOQA
 
-try:
-    import kvxopt
-    from kvxopt import umfpack   # test if shared libs can be found
-    from kvxopt import spmatrix as kspmatrix
-    KIP_ADD = True
-except ImportError:
-    kvxopt = None
-    kspmatrix = None
-    KIP_ADD = False
-
-if KIP_ADD is False:
-    from cvxopt import spmatrix as cspmatrix
-    if hasattr(cspmatrix, 'ipadd'):
-        CIP_ADD = True
-    else:
-        CIP_ADD = False
-
-
-if kvxopt is None or (KIP_ADD is False and CIP_ADD is True):
-    from cvxopt import umfpack                           # NOQA
-    from cvxopt import spmatrix, matrix, sparse, spdiag  # NOQA
-    from cvxopt import mul, div                          # NOQA
-    from cvxopt.lapack import gesv                       # NOQA
-    from cvxopt import printing                          # NOQA
-    klu = None
-    IP_ADD = CIP_ADD
-else:
-    from kvxopt import umfpack, klu                      # NOQA
-    from kvxopt import spmatrix, matrix, sparse, spdiag  # NOQA
-    from kvxopt import mul, div                          # NOQA
-    from kvxopt.lapack import gesv                       # NOQA
-    from kvxopt import printing                          # NOQA
-    IP_ADD = KIP_ADD
+from andes.utils.lazyimport import LazyImport
 
 printing.options['dformat'] = '%.1f'
 printing.options['width'] = -1
 
-from andes.utils.texttable import Texttable              # NOQA
-from andes.utils.paths import get_dot_andes_path         # NOQA
+from andes.utils.paths import get_dot_andes_path  # NOQA
+from andes.utils.texttable import Texttable  # NOQA
 
 # --- constants ---
 
 deg2rad = math.pi/180
+pi2o3 = math.pi * 2 / 3
+sqrt3 = math.sqrt(3)
+isqrt3 = math.sqrt(1/3)
+
 jac_names = ('fx', 'fy', 'gx', 'gy')
 jac_types = ('c', '')
 
@@ -81,6 +53,7 @@ for jname in jac_names:
 tqdm = LazyImport('from tqdm import tqdm')
 
 pd = LazyImport('import pandas')
+numba = LazyImport('import numba')
 cupy = LazyImport('import cupy')
 mpl = LazyImport('import matplotlib')
 unittest = LazyImport('import unittest')
