@@ -1,12 +1,12 @@
 import logging
-import numpy as np
-
-from typing import List, Union
 from collections import OrderedDict
+from typing import List, Union
+
+import numpy as np
 
 from andes.core import JacTriplet
 from andes.core.var import BaseVar
-from andes.shared import pd, spmatrix, jac_names
+from andes.shared import jac_names, pd, spmatrix
 
 logger = logging.getLogger(__name__)
 
@@ -146,6 +146,9 @@ class DAETimeSeries:
 
         return super().__getattribute__(attr)
 
+    def __getstate__(self):
+        return self.__dict__
+
 
 class DAE:
     r"""
@@ -202,7 +205,7 @@ class DAE:
 
     def __init__(self, system):
         self.system = system
-        self.t = np.array(0)
+        self.t = np.array(0, dtype=float)
         self.ts = DAETimeSeries(self)
 
         self.m, self.n, self.o = 0, 0, 0
